package szakdolgozat;

import java.awt.Color;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.AbstractAction;
import javax.swing.ActionMap;
import javax.swing.GroupLayout;
import javax.swing.GroupLayout.ParallelGroup;
import javax.swing.GroupLayout.SequentialGroup;
import javax.swing.InputMap;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.LayoutStyle;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultHighlighter;
import javax.swing.text.Highlighter;

public class HelpPanel extends JPanel implements DocumentListener {

	/**
	 * 
	 */
	private static final long serialVersionUID = 5512906520911309102L;
	private JTextField entry;
	private JLabel jLabel1;
	private JScrollPane jScrollPane1;
	private JLabel status;
	private JTextArea textArea;

	final static Color HILIT_COLOR = Color.LIGHT_GRAY;
	final static Color ACTUAL_HILIT_COLOR = Color.ORANGE;
	final static Color ERROR_COLOR = Color.PINK;
	final static String CANCEL_ACTION = "cancel-search";

	final Color entryBg;
	final Highlighter hilit;
	final Highlighter.HighlightPainter painter;
	final Highlighter.HighlightPainter actualPainter;
	private JButton button;
	private final ActionListener next = new NextSearch();
	private int last;
	private Matcher matcher;

	public HelpPanel() {
		initComponents();
		InputStream in = ClassLoader.getSystemClassLoader().getResourceAsStream("help.txt");
		try {
			BufferedReader br = new BufferedReader(new InputStreamReader(in,"UTF-8"));
			String sr = "";
			String line;
			while ((line = br.readLine()) != null) {
				sr += line;
				sr += '\n';
			}
			textArea.setText(sr);
			textArea.setFont(new Font("Verdana",Font.ROMAN_BASELINE,14));

		} catch (IOException e) {
			e.printStackTrace();
		}

		hilit = new DefaultHighlighter();
		painter = new DefaultHighlighter.DefaultHighlightPainter(HILIT_COLOR);
		textArea.setHighlighter(hilit);
		actualPainter = new DefaultHighlighter.DefaultHighlightPainter(ACTUAL_HILIT_COLOR);
		//        textArea.setHighlighter(actualHilit);
		entryBg = entry.getBackground();
		entry.getDocument().addDocumentListener(this);
		textArea.setCaretPosition(0);
		InputMap im = entry.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
		ActionMap am = entry.getActionMap();
		im.put(KeyStroke.getKeyStroke("ESCAPE"), CANCEL_ACTION);
		am.put(CANCEL_ACTION, new CancelAction());
	}

	/**
	 * This method is called from within the constructor to initialize the form.
	 */

	private void initComponents() {
		entry = new JTextField();
		textArea = new JTextArea();
		status = new JLabel();
		jLabel1 = new JLabel();
		button = new JButton();
		button.setText("Következő");

		textArea.setColumns(20);
		textArea.setLineWrap(true);
		textArea.setRows(5);
		textArea.setWrapStyleWord(true);
		textArea.setEditable(false);
		jScrollPane1 = new JScrollPane(textArea);
		button.setText("Keresés");

		jLabel1.setText("Adja meg a keresni kívánt szöveget.");

		GroupLayout layout = new GroupLayout(this);
		setLayout(layout);

		// Create a parallel group for the horizontal axis
		ParallelGroup hGroup = layout.createParallelGroup(GroupLayout.Alignment.LEADING);

		SequentialGroup h1 = layout.createSequentialGroup();
		ParallelGroup h2 = layout.createParallelGroup(GroupLayout.Alignment.TRAILING);

		h1.addContainerGap();

		h2.addComponent(jScrollPane1, GroupLayout.Alignment.LEADING, GroupLayout.DEFAULT_SIZE, 450, Short.MAX_VALUE);
		h2.addComponent(status, GroupLayout.Alignment.LEADING, GroupLayout.DEFAULT_SIZE, 450, Short.MAX_VALUE);

		SequentialGroup h3 = layout.createSequentialGroup();
		h3.addComponent(jLabel1);
		h3.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED);
		h3.addComponent(entry, GroupLayout.DEFAULT_SIZE, 321, Short.MAX_VALUE);
		h3.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED);
		h3.addComponent(button);

		h2.addGroup(h3);

		h1.addGroup(h2);

		h1.addContainerGap();

		hGroup.addGroup(GroupLayout.Alignment.TRAILING, h1);

		layout.setHorizontalGroup(hGroup);

		ParallelGroup vGroup = layout.createParallelGroup(GroupLayout.Alignment.LEADING);

		SequentialGroup v1 = layout.createSequentialGroup();

		v1.addContainerGap();

		ParallelGroup v2 = layout.createParallelGroup(GroupLayout.Alignment.BASELINE);
		v2.addComponent(jLabel1);
		v2.addComponent(entry, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE);

		v2.addComponent(button);
		v1.addGroup(v2);
		v1.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED);
		v1.addComponent(jScrollPane1, GroupLayout.DEFAULT_SIZE, 233, Short.MAX_VALUE);
		v1.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED);
		v1.addComponent(status);
		v1.addContainerGap();
		button.addActionListener(next);

		vGroup.addGroup(v1);
		// Create the vertical group
		layout.setVerticalGroup(vGroup);
	}

	public void search() {
		hilit.removeAllHighlights();
		last = 0;
		int elso = -1;
		String s = entry.getText();
		if (s.length() <= 0) {
			message("Nincs keresési feltétel.");
		} else {
			Pattern pattern = Pattern.compile(s, Pattern.CASE_INSENSITIVE);
			String content = textArea.getText();
			matcher = pattern.matcher(content);
			while (matcher.find()) {
				int index = matcher.start();
				elso = elso == -1 ? index : elso;
				int end = matcher.end();
				addHighLight(hilit, index, end, painter);
				entry.setBackground(entryBg);
			}
			if (elso != -1) {
				textArea.setCaretPosition(elso);
			} else {
				message("Nincs találat!");
			}
		}
	}

	void message(final String msg) {
		status.setText(msg);
	}


	@Override
	public void insertUpdate(final DocumentEvent ev) {
		search();
	}

	@Override
	public void removeUpdate(final DocumentEvent ev) {
		search();
	}

	@Override
	public void changedUpdate(final DocumentEvent ev) {
	}

	class CancelAction extends AbstractAction {
		/**
		 * 
		 */
		private static final long serialVersionUID = 7791101757808282016L;

		@Override
		public void actionPerformed(final ActionEvent ev) {
			hilit.removeAllHighlights();
			entry.setText("");
			entry.setBackground(entryBg);
		}
	}

	class NextSearch implements ActionListener {

		@Override
		public void actionPerformed(final ActionEvent e) {
			if (matcher != null && matcher.find(last)) {
				addHighLight(hilit, matcher.start(), matcher.end(), actualPainter);
				textArea.setCaretPosition(matcher.start());
				last = matcher.end();
			} else {
				message("Nincs több találat!");
			}
		}

	}

	private void addHighLight(final Highlighter highlighter, final int start, final int end, final Highlighter.HighlightPainter painter) {
		try {
			highlighter.addHighlight(start, end, painter);
		} catch (BadLocationException e) {
			e.printStackTrace();
		}
	}
}